#include "cnoise.h"
#include <time.h>
#include <stdio.h>

#define CNOISE_EPSILON (0.00005f)

//---------------------------------------------------------------------------------------
CNoise::CNoise()
{
   mSelectMode = ac_get_select_mode();
   // Seed randomizer with current date/time
   srand(( unsigned int )time( NULL ));
}
//---------------------------------------------------------------------------------------
CNoise::~CNoise()
{
}
//---------------------------------------------------------------------------------------
void CNoise::AddUndoVertPos()
{

   switch( mSelectMode )
   {
   case SELECT_GROUP:
      {
         List *vlist = NULL;
         List *olist = ac_selection_get_objects();
         AddGroupUndoPos( &vlist, olist );
         list_free( &olist );
         if( vlist != NULL )
         {
            add_undoable_vertex_positions( "noise", vlist );
            list_free( &vlist );
         }
      }
      break;
   case SELECT_OBJECT:
      {
         List *vlist = NULL;
         List *olist = ac_selection_get_objects();
         for( List *p = olist; p != NULL; p = p->next )
         {
            list_append_list( &vlist, list_clone( ac_object_get_vertexlist(( ACObject * )( p->data ))));
         }
         list_free( &olist );
         if( vlist != NULL )
         {
            add_undoable_vertex_positions( "noise", vlist );
            list_free( &vlist );
         }
      }
      break;
   case SELECT_SURFACE:
      {
         List *vlist = NULL;
         List *slist = ac_selection_get_surfaces();
         for( List *p = slist; p != NULL; p = p->next )
         {
            Surface *s = ( Surface * )( p->data );
            for( List *pv = s->vertlist; pv != NULL; pv = pv->next )
            {
               list_add_item_head( &vlist, (( SVertex * )( pv->data ))->v );
            }
         }
         list_free( &slist );
         if( vlist != NULL )
         {
            add_undoable_vertex_positions( "noise", vlist );
            list_free( &vlist );
         }
      }
      break;
   case SELECT_VERTEX:
      {
         List *vlist = ac_selection_get_vertices();
         if( vlist != NULL )
         {
            add_undoable_vertex_positions( "noise", vlist );
            list_free( &vlist );
         }
      }
      break;
   }

}
//---------------------------------------------------------------------------------------
void CNoise::AddUndoSurfTex()
{

   switch( mSelectMode )
   {
   case SELECT_GROUP:
      {
         List *slist = NULL;
         List *olist = ac_selection_get_objects();
         AddGroupUndoTex( &slist, olist );
         list_free( &olist );
         if( slist != NULL )
         {
            add_undo_texture_coords( "noise", slist, NULL );
            list_free( &slist );
         }
      }
      break;
   case SELECT_OBJECT:
      {
         List *slist = NULL;
         List *olist = ac_selection_get_objects();
         for( List *p = olist; p != NULL; p = p->next )
         {
            list_append_list( &slist, list_clone( ac_object_get_surfacelist(( ACObject * )( p->data ))));
         }
         list_free( &olist );
         if( slist != NULL )
         {
            add_undo_texture_coords( "noise", slist, NULL );
            list_free( &slist );
         }
      }
      break;
   case SELECT_SURFACE:
      {
         List *slist = ac_selection_get_surfaces();
         add_undo_texture_coords( "noise", slist, NULL );
         list_free( &slist );
      }
      break;
   case SELECT_VERTEX:
      {
         List *vlist = ac_selection_get_vertices();
         List *slist = NULL;
         for( List *p = vlist; p != NULL; p = p->next )
         {
            list_append_list( &slist, list_clone( surfaces_of_vertex(( Vertex * )( p->data ))));
         }
         list_free( &vlist );
         if( slist != NULL )
         {
            add_undo_texture_coords( "noise", slist, NULL );
            list_free( &slist );
         }
      }
      break;
   }

}
//---------------------------------------------------------------------------------------
void CNoise::MoveGroupVertices( ACObject *o, Vertex &delta )
{

   List *vlist = ac_object_get_vertexlist( o );

   for( List *pv = vlist; pv != NULL; pv = pv->next )
   {
      Vertex *v = ( Vertex * )( pv->data );
      v->x += delta.x;
      v->y += delta.y;
      v->z += delta.z;
   }

   for( List *p = ac_object_get_childrenlist( o ); p != NULL; p = p->next )
   {
      MoveGroupVertices(( ACObject * )( p->data ), delta );
   }

}
//---------------------------------------------------------------------------------------
void CNoise::Move( Vertex &opvec )
{

   switch( mSelectMode )
   {
   case SELECT_GROUP:
      {
         List *olist = ac_selection_get_objects();
         Vertex delta;
         for( List *p = olist; p != NULL; p = p->next )
         {
            delta.x = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.x;
            delta.y = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.y;
            delta.z = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.z;
            MoveGroupVertices(( ACObject * )( p->data ), delta );
         }
         list_free( &olist );
      }
      break;
   case SELECT_OBJECT:
      {
         List *olist = ac_selection_get_objects();
         Vertex delta;
         for( List *p = olist; p != NULL; p = p->next )
         {
            delta.x = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.x;
            delta.y = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.y;
            delta.z = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.z;
            for( List *pv = ac_object_get_vertexlist(( ACObject * )( p->data )); pv != NULL; pv = pv->next )
            {
               Vertex *v = ( Vertex * )( pv->data );
               v->x += delta.x;
               v->y += delta.y;
               v->z += delta.z;
            }
         }
         list_free( &olist );
      }
      break;
   case SELECT_SURFACE:
      {
         List *slist = ac_selection_get_surfaces();
         Vertex delta;
         for( List *p = slist; p != NULL; p = p->next )
         {
            Surface *s = ( Surface * )( p->data );
            delta.x = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.x;
            delta.y = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.y;
            delta.z = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.z;
            for( List *pv = s->vertlist; pv != NULL; pv = pv->next )
            {
               SVertex *v = ( SVertex * )( pv->data );
               v->v->x += delta.x;
               v->v->y += delta.y;
               v->v->z += delta.z;
            }
         }
      }
      break;
   case SELECT_VERTEX:
      {
         List *vlist = ac_selection_get_vertices();
         for( List *p = vlist; p != NULL; p = p->next )
         {
            Vertex *v = ( Vertex * )( p->data );
            v->x += (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.x;
            v->y += (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.y;
            v->z += (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.z;
         }
         list_free( &vlist );
      }
      break;
   }

}
//---------------------------------------------------------------------------------------
void CNoise::ScaleGroupVertices( ACObject *o, Point3 *center, Point3 *size, Vertex &delta )
{

   List *vlist = ac_object_get_vertexlist( o );

   for( List *pv = vlist; pv != NULL; pv = pv->next )
   {
      Vertex *v = ( Vertex * )( pv->data );
printf( "===============================\nv = %g\nc = %g\ns = %g\nd = %g\n",
       v->x, center->x, size->x, delta.x );
      if( size->x > 0.0f ) v->x += (( v->x - center->x ) / size->x ) * delta.x;
      if( size->y > 0.0f ) v->y += (( v->y - center->y ) / size->y ) * delta.y;
      if( size->z > 0.0f ) v->z += (( v->z - center->z ) / size->z ) * delta.z;
   }

   for( List *p = ac_object_get_childrenlist( o ); p != NULL; p = p->next )
   {
      ScaleGroupVertices(( ACObject * )( p->data ), center, size, delta );
   }

}
//---------------------------------------------------------------------------------------
void CNoise::Scale( Vertex &opvec )
{

   switch( mSelectMode )
   {
   case SELECT_GROUP:
      {
         List *olist = ac_selection_get_objects();
         Vertex delta;
         Point3 center, size;
         for( List *p = olist; p != NULL; p = p->next )
         {
            delta.x = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.x;
            delta.y = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.y;
            delta.z = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.z;
            delta.x = opvec.x;
            delta.y = opvec.y;
            delta.z = opvec.z;
            ACObject *o = ( ACObject * )( p->data );
            Point3 omax;
            Point3 omin;
            if( ac_object_find_bound( o, &omax, &omin ) != 0 )
            {
//printf( "%g -> %g -> %g\n", omin.x, omax.x, size.x );
               size.x = ( omax.x - omin.x ) * 0.5f;
               size.y = ( omax.y - omin.y ) * 0.5f;
               size.z = ( omax.z - omin.z ) * 0.5f;
               center.x = omin.x + size.x;
               center.y = omin.y + size.y;
               center.z = omin.z + size.z;
               ScaleGroupVertices( o, &center, &size, delta );
            }
         }
         list_free( &olist );
      }
      break;
   case SELECT_OBJECT:
      {
         List *olist = ac_selection_get_objects();
         Vertex delta;
         for( List *p = olist; p != NULL; p = p->next )
         {
            delta.x = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.x;
            delta.y = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.y;
            delta.z = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.z;
            ACObject *o = ( ACObject * )( p->data );
            Point3 *center = ac_object_get_centre( o );
            for( List *pv = ac_object_get_vertexlist(( ACObject * )( p->data )); pv != NULL; pv = pv->next )
            {
               C3d::RotatePoint(( Vertex * )( pv->data ), center, &delta );
            }
         }
         list_free( &olist );
      }
      break;
   case SELECT_SURFACE:
      {
         List *slist = ac_selection_get_surfaces();
         Vertex delta;
         for( List *p = slist; p != NULL; p = p->next )
         {
            Surface *s = ( Surface * )( p->data );
            delta.x = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.x;
            delta.y = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.y;
            delta.z = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.z;
            Point3 *center = ac_object_get_centre( object_of_surface( s ));
            for( List *pv = s->vertlist; pv != NULL; pv = pv->next )
            {
               C3d::RotatePoint((( SVertex * )( pv->data ))->v, center, &delta );
            }
         }
      }
      break;
   case SELECT_VERTEX:
      {
         List *vlist = ac_selection_get_vertices();
         Vertex delta;
         for( List *p = vlist; p != NULL; p = p->next )
         {
            delta.x = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.x;
            delta.y = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.y;
            delta.z = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.z;
            Vertex *v = ( Vertex * )( p->data );
            Point3 *center = ac_object_get_centre( object_of_vertex( v ));
            C3d::RotatePoint( v, center, &delta );
         }
         list_free( &vlist );
      }
      break;
   }

}
//---------------------------------------------------------------------------------------
void CNoise::RotateGroupVertices( ACObject *o, Point3 *center, Vertex &delta )
{

   List *vlist = ac_object_get_vertexlist( o );

   for( List *pv = vlist; pv != NULL; pv = pv->next )
   {
      C3d::RotatePoint(( Vertex * )( pv->data ), center, &delta );
   }

   for( List *p = ac_object_get_childrenlist( o ); p != NULL; p = p->next )
   {
      RotateGroupVertices(( ACObject * )( p->data ), center, delta );
   }

}
//---------------------------------------------------------------------------------------
void CNoise::Rotate( Vertex &opvec )
{

   switch( mSelectMode )
   {
   case SELECT_GROUP:
      {
         List *olist = ac_selection_get_objects();
         Vertex delta;
         for( List *p = olist; p != NULL; p = p->next )
         {
            delta.x = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.x;
            delta.y = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.y;
            delta.z = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.z;
            ACObject *o = ( ACObject * )( p->data );
            Point3 *center = ac_object_get_centre( o );
            RotateGroupVertices( o, center, delta );
         }
         list_free( &olist );
      }
      break;
   case SELECT_OBJECT:
      {
         List *olist = ac_selection_get_objects();
         Vertex delta;
         for( List *p = olist; p != NULL; p = p->next )
         {
            delta.x = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.x;
            delta.y = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.y;
            delta.z = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.z;
            ACObject *o = ( ACObject * )( p->data );
            Point3 *center = ac_object_get_centre( o );
            for( List *pv = ac_object_get_vertexlist(( ACObject * )( p->data )); pv != NULL; pv = pv->next )
            {
               C3d::RotatePoint(( Vertex * )( pv->data ), center, &delta );
            }
         }
         list_free( &olist );
      }
      break;
   case SELECT_SURFACE:
      {
         List *slist = ac_selection_get_surfaces();
         Vertex delta;
         for( List *p = slist; p != NULL; p = p->next )
         {
            Surface *s = ( Surface * )( p->data );
            delta.x = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.x;
            delta.y = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.y;
            delta.z = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.z;
            Point3 *center = ac_object_get_centre( object_of_surface( s ));
            for( List *pv = s->vertlist; pv != NULL; pv = pv->next )
            {
               C3d::RotatePoint((( SVertex * )( pv->data ))->v, center, &delta );
            }
         }
      }
      break;
   case SELECT_VERTEX:
      {
         List *vlist = ac_selection_get_vertices();
         Vertex delta;
         for( List *p = vlist; p != NULL; p = p->next )
         {
            delta.x = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.x;
            delta.y = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.y;
            delta.z = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.z;
            Vertex *v = ( Vertex * )( p->data );
            Point3 *center = ac_object_get_centre( object_of_vertex( v ));
            C3d::RotatePoint( v, center, &delta );
         }
         list_free( &vlist );
      }
      break;
   }

}
//---------------------------------------------------------------------------------------
void CNoise::AddGroupUndoPos( List **vlist, List *olist )
{
   for( List *p = olist; p != NULL; p = p->next )
   {
      ACObject *o = ( ACObject * )( p->data );
      List *lt = ac_object_get_vertexlist( o );
      if( lt != NULL )
      {
         list_append_list( vlist, list_clone( lt ));
      }
      lt = ac_object_get_childrenlist( o );
      if( lt != NULL )
      {
         AddGroupUndoPos( vlist, lt );
      }
   }
}
//---------------------------------------------------------------------------------------
void CNoise::AddGroupUndoTex( List **slist, List *olist )
{
   for( List *p = olist; p != NULL; p = p->next )
   {
      ACObject *o = ( ACObject * )( p->data );
      List *lt = ac_object_get_surfacelist( o );
      if( lt != NULL )
      {
         list_append_list( slist, list_clone( lt ));
      }
      lt = ac_object_get_childrenlist( o );
      if( lt != NULL )
      {
         AddGroupUndoTex( slist, lt );
      }
   }
}
//---------------------------------------------------------------------------------------
void CNoise::TexCoordGroupVertices( ACObject *o, Point2 &delta )
{

   List *slist = ac_object_get_surfacelist( o );

   for( List *ps = slist; ps != NULL; ps = ps->next )
   {
      Surface *s = ( Surface * )( ps->data );
      for( List *psv = s->vertlist; psv != NULL; psv = psv->next )
      {
         SVertex *sv = ( SVertex * )( psv->data );
         sv->tx += delta.x;
         sv->ty += delta.y;
      }
   }

   for( List *p = ac_object_get_childrenlist( o ); p != NULL; p = p->next )
   {
      TexCoordGroupVertices(( ACObject * )( p->data ), delta );
   }

}
//---------------------------------------------------------------------------------------
void CNoise::TexCoord( Vertex &opvec )
{

   switch( mSelectMode )
   {
   case SELECT_GROUP:
      {
         List *olist = ac_selection_get_objects();
         Point2 delta;
         for( List *p = olist; p != NULL; p = p->next )
         {
            delta.x = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.x;
            delta.y = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.y;
            TexCoordGroupVertices(( ACObject * )( p->data ), delta );
         }
         list_free( &olist );
      }
      break;
   case SELECT_OBJECT:
      {
         List *olist = ac_selection_get_objects();
         Point2 delta;
         for( List *p = olist; p != NULL; p = p->next )
         {
            delta.x = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.x;
            delta.y = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.y;
            for( List *ps = ac_object_get_surfacelist(( ACObject * )( p->data )); ps != NULL; ps = ps->next )
            {
               Surface *s = ( Surface * )( ps->data );
               for( List *psv = s->vertlist; psv != NULL; psv = psv->next )
               {
                  SVertex *sv = ( SVertex * )( psv->data );
                  sv->tx += delta.x;
                  sv->ty += delta.y;
               }
            }
         }
         list_free( &olist );
      }
      break;
   case SELECT_SURFACE:
      {
         List *slist = ac_selection_get_surfaces();
         Point2 delta;
         for( List *p = slist; p != NULL; p = p->next )
         {
            Surface *s = ( Surface * )( p->data );
            delta.x = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.x;
            delta.y = (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.y;
            for( List *pv = s->vertlist; pv != NULL; pv = pv->next )
            {
               SVertex *v = ( SVertex * )( pv->data );
               v->tx += delta.x;
               v->ty += delta.y;
            }
         }
      }
      break;
   case SELECT_VERTEX:
      {
         List *vlist = ac_selection_get_vertices();
         for( List *p = vlist; p != NULL; p = p->next )
         {
            Vertex *v = ( Vertex * )( p->data );
            v->x += (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.x;
            v->y += (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.y;
            v->z += (( float )( rand() % 10000 - 5000 )) * 0.0001f * opvec.z;
         }
         list_free( &vlist );
      }
      break;
   }

}
//---------------------------------------------------------------------------------------

#undef CNOISE_EPSILON
