#include "cnoise.h"
#include <stdio.h>
#include <string.h>

//---------------------------------------------------------------------------------------
// Move/rotate objects
static void ddhnoise_go( int argc, const char* argv[] )
{

   if( argc < 5 )
   {
      message_dialog( "Insufficient parameters passed to noise plugin" );
      return;
   }

   // Get first 3 floats
   Vertex opvec;
   if( sscanf( argv[1], "%g", &( opvec.x )) != 1 )
   {
      message_dialog( "Unable to parse value for X (%s)", argv[1] );
   }
   if( sscanf( argv[2], "%g", &( opvec.y )) != 1 )
   {
      message_dialog( "Unable to parse value for Y (%s)", argv[2] );
   }
   if( sscanf( argv[3], "%g", &( opvec.z )) != 1 )
   {
      message_dialog( "Unable to parse value for Z (%s)", argv[3] );
   }

   // Get the operation
   // 1 = rotate
   // 2 = scale
   // 3 = texture coordinates
   // any other = move
   int operation = atoi( argv[4] );

   if( operation == 3 && ac_get_select_mode() == SELECT_VERTEX )
   {
      message_dialog( "Texture coordinates are not supported in Vertex select mode" );
      return;
   }

   CNoise noise;

   switch( operation )
   {
   case 1:
      noise.AddUndoVertPos();
      opvec.x = DEGTORAD( opvec.x );
      opvec.y = DEGTORAD( opvec.y );
      opvec.z = DEGTORAD( opvec.z );
      noise.Rotate( opvec );
      selected_calc_normals_force();
      find_bounding_box();
      break;
   case 2:
      noise.AddUndoVertPos();
      opvec.x *= 0.5f;
      opvec.y *= 0.5f;
      opvec.z *= 0.5f;
      noise.Scale( opvec );
      selected_calc_normals_force();
      find_bounding_box();
      break;
   case 3:
      noise.AddUndoSurfTex();
      opvec.x *= 0.01f;
      opvec.y *= 0.01f;
      noise.TexCoord( opvec );
      tcl_command( "ac3d remake_subdivisions" );
      break;
   default:
      noise.AddUndoVertPos();
      noise.Move( opvec );
      selected_calc_normals_force();
      find_bounding_box();
      break;
   }

   redraw_all();
   display_status();

}
//---------------------------------------------------------------------------------------
AC3D_PLUGIN_FUNC int AC3DPluginInit( AC3DPluginInitData *d )
{

   static bool firstTime = true;
   if( firstTime )
   {

      char *path = ( char * )strdup( d->plugin_dir_path );
#ifdef WINDOWS
      ac_unbackslash_string( path );
#endif

      tcl_command( "source \"%s/noise.tcl\"", path );
      ac_add_tools_menu_item( "Noise...", "ddhnoise_dialog",
                              "Randomly move or rotate geometry" );
      ac_add_command_full( "ddhnoise_go", ( void * )ddhnoise_go,
                            0, "argv", "<arg1> <arg2>",
                           "Randomly move or rotate geometry" );

      firstTime = false;

   }

   return 0;

}
//---------------------------------------------------------------------------------------
AC3D_PLUGIN_FUNC int AC3DPluginExit()
{
   return 0;
}
//---------------------------------------------------------------------------------------
AC3D_PLUGIN_FUNC char *AC3DPluginAbout()
{
   return( "Noise - version 1.0.1 - Dennis Hawthorne" );
}
//---------------------------------------------------------------------------------------
AC3D_PLUGIN_FUNC char *AC3DPluginInfo()
{
   return( "Noise - Dennis Hawthorne - Randomly move or rotate geometry." );
}
//---------------------------------------------------------------------------------------
